class JobList extends RessourceList {
  static autoInit() {
    for (let jobListElement of document.querySelectorAll('.job-list:not(.no-autoinit)')) {
      new JobList(jobListElement);
    }
  }

  static options = {
    listContainerInnerHTMLGenerator: (listContainerElement) => {
      listContainerElement.innerHTML = `
        <div class="input-field">
          <i class="material-icons prefix">search</i>
          <input id="${listContainerElement.id}-search" class="search" type="text"></input>
          <label for="${listContainerElement.id}-search">Search job</label>
        </div>
        <table>
          <thead>
            <tr>
              <th>Service</th>
              <th>Title and Description</th>
              <th>Status</th>
              <th></th>
            </tr>
          </thead>
          <tbody class="list"></tbody>
        </table>
        <ul class="pagination"></ul>
      `.trim();
    },
    ressourceMapper: (job) => {
      return {
        'id': job.id,
        'creation-date': job.creation_date,
        'description': job.description,
        'service': job.service,
        'status': job.status,
        'title': job.title
      };
    },
    sortParams: ['creation-date', {order: 'desc'}],
    listjs: {
      item: `
        <tr class="clickable hoverable service-scheme">
          <td><a class="btn-floating"><i class="nopaque-icons service-icons" data-service="inherit"></i></a></td>
          <td><b class="title"></b><br><i class="description"></i></td>
          <td><span class="badge new job-status-color job-status-text status" data-badge-caption=""></span></td>
          <td class="right-align">
            <a class="action-button btn-floating red waves-effect waves-light" data-action="delete-request"><i class="material-icons">delete</i></a>
            <a class="action-button btn-floating darken waves-effect waves-light" data-action="view"><i class="material-icons">send</i></a>
          </td>
        </tr>
      `.trim(),
      valueNames: [
        {data: ['id']},
        {data: ['creation-date']},
        {data: ['service']},
        {name: 'status', attr: 'data-status'},
        'description',
        'title'
      ]
    }
  };

  constructor(listContainerElement, options={}) {
    super(listContainerElement, _.merge({}, JobList.options, options));
    console.log(this);
  }

  init(user) {
    this._init(user.jobs);
  }

  onClick(event) {
    let jobElement = event.target.closest('tr');
    if (jobElement === null) {return;}
    let jobId = jobElement.dataset.id;
    if (jobId === undefined) {return;}
    let actionButtonElement = event.target.closest('.action-button');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete-request': {
        Utils.deleteJobRequest(this.userId, jobId);
        break;
      }
      case 'view': {
        window.location.href = `/jobs/${jobId}`;
        break;
      }
      default: {
        break;
      }
    }
  }

  onPatch(patch) {
    let re = new RegExp(`^/users/${this.userId}/jobs/([A-Za-z0-9]*)`);
    let filteredPatch = patch.filter(operation => re.test(operation.path));
    for (let operation of filteredPatch) {
      switch(operation.op) {
        case 'add': {
          let re = new RegExp(`^/users/${this.userId}/jobs/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        }
        case 'remove': {
          let re = new RegExp(`^/users/${this.userId}/jobs/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {
            let [match, jobId] = operation.path.match(re);
            this.remove(jobId);
          }
          break;
        }
        case 'replace': {
          let re = new RegExp(`^/users/${this.userId}/jobs/([A-Za-z0-9]*)/(service|status|description|title)$`);
          if (re.test(operation.path)) {
            let [match, jobId, valueName] = operation.path.match(re);
            this.replace(jobId, valueName, operation.value);
          }
          break;
        }
        default: {
          break;
        }
      }
    }
  }
}
