class UserList extends RessourceList {
  static autoInit() {
    for (let userListElement of document.querySelectorAll('.user-list:not(.no-autoinit)')) {
      new UserList(userListElement);
    }
  }

  static options = {
    listContainerInnerHTMLGenerator: (listContainerElement) => {
      listContainerElement.innerHTML = `
        <div class="input-field">
          <i class="material-icons prefix">search</i>
          <input id="${listContainerElement.id}-search" class="search" type="text"></input>
          <label for="${listContainerElement.id}-search">Search user</label>
        </div>
        <table>
          <thead>
            <tr>
              <th>Id</th>
              <th>Username</th>
              <th>Email</th>
              <th>Last seen</th>
              <th>Role</th>
              <th></th>
            </tr>
          </thead>
          <tbody class="list"></tbody>
        </table>
        <ul class="pagination"></ul>
      `.trim();
    },
    ressourceMapper: (user) => {
      return {
        'id': user.id,
        'id-1': user.id,
        'username': user.username,
        'email': user.email,
        'last-seen': new Date(user.last_seen).toLocaleString('en-US'),
        'member-since': user.member_since,
        'role': user.role.name
      };
    },
    sortParams: ['member-since', {order: 'desc'}],
    listjs: {
      item: `
        <tr class="clickable hoverable">
          <td><span class="id-1"></span></td>
          <td><span class="username"></span></td>
          <td><span class="email"></span></td>
          <td><span class="last-seen"></span></td>
          <td><span class="role"></span></td>
          <td class="right-align">
            <a class="action-button btn-floating red waves-effect waves-light" data-action="delete"><i class="material-icons">delete</i></a>
            <a class="action-button btn-floating waves-effect waves-light" data-action="edit"><i class="material-icons">edit</i></a>
            <a class="action-button btn-floating waves-effect waves-light" data-action="view"><i class="material-icons">send</i></a>
          </td>
        </tr>
      `.trim(),
      valueNames: [
        {data: ['id']},
        {data: ['member-since']},
        'email',
        'id-1',
        'last-seen',
        'role',
        'username'
      ]
    }
  };

  constructor(listContainerElement, options = {}) {
    super(listContainerElement, {...UserList.options, ...options});
  }

  init(users) {
    super._init(Object.values(users));
  }

  onClick(event) {
    let userElement = event.target.closest('tr');
    if (userElement === null) {return;}
    let userId = userElement.dataset.id;
    if (userId === undefined) {return;}
    let actionButtonElement = event.target.closest('.action-button');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete': {
        Utils.deleteUserRequest(userId);
        if (userId === currentUserId) {window.location.href = '/';}
        break;
      }
      case 'edit': {
        window.location.href = `/admin/users/${userId}/edit`;
        break;
      }
      case 'view': {
        window.location.href = `/admin/users/${userId}`;
        break;
      }
      default: {
        break;
      }
    }
  }
}
